import time

from iwla import IWLA
from iplugin import IPlugin
from display import *

class IWLADisplayTopHits(IPlugin):
    def __init__(self, iwla):
        super(IWLADisplayTopHits, self).__init__(iwla)
        self.API_VERSION = 1
        self.requires = ['IWLAPostAnalysisTopHits']

    def hook(self):
        top_hits = self.iwla.getMonthStats()['top_hits']

        top_hits = sorted(top_hits.items(), key=lambda t: t[1], reverse=True)

        index = self.iwla.getDisplayIndex()
        
        table = DisplayHTMLBlockTable('Top Hits', ['URI', 'Entrance'])        
        for (uri, entrance) in top_hits[:10]:
            table.appendRow([uri, entrance])
        index.appendBlock(table)

        title = time.strftime('All Hits - %B %Y', self.iwla.getCurTime())

        filename = 'top_hits.html'
        path = self.iwla.getCurDisplayPath(filename)

        page = DisplayHTMLPage(title, path)
        table = DisplayHTMLBlockTable('Top Hits', ['URI', 'Entrance'])        
        for (uri, entrance) in top_hits:
            table.appendRow([uri, entrance])
        page.appendBlock(table)
                        
        display = self.iwla.getDisplay()
        display.addPage(page)

        block = DisplayHTMLRawBlock()
        block.setRawHTML('<a href=\'%s\'>All hits</a>' % (filename))
        index.appendBlock(block)
